@@ -1,10 +1,10 @@
 package Net::DNS::Domain;
 
 #
-# $Id: Domain.pm 1158 2014-01-10 22:30:40Z willem $
+# $Id: Domain.pm 1183 2014-04-03 09:08:55Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1158 $)[1];
+$VERSION = (qw$LastChangedRevision: 1183 $)[1];
 
 
 =head1 NAME
@@ -43,12 +43,14 @@ use Carp;
 
 use constant ASCII => eval {
 	require Encode;
-	Encode::find_encoding('ASCII');				# return encoding object
+	Encode::find_encoding('ascii');				# encoding object
+	1;
 } || 0;
 
 use constant UTF8 => eval {
 	die if Encode::decode_utf8( chr(91) ) ne '[';		# not UTF-EBCDIC  [see UTR#16 3.6]
-	Encode::find_encoding('UTF8');				# return encoding object
+	Encode::find_encoding('utf8');				# encoding object
+	1;
 } || 0;
 
 use constant LIBIDN => eval {
@@ -57,6 +59,14 @@ use constant LIBIDN => eval {
 } || 0;
 
 
+my ( $ascii, $utf8 );			## perlcc: initialisation deferred until object creation
+
+sub init {
+	$ascii = Encode::find_encoding('ascii') if ASCII;	# Osborn's Law:
+	$utf8  = Encode::find_encoding('utf8')	if UTF8;	# Variables won't; constants aren't.
+}
+
+
 =head1 METHODS
 
 =head2 new
@@ -85,21 +95,20 @@ for zone files described in RFC1035.
 =cut
 
 use vars qw($ORIGIN);
-
-my $cache1 = {};
-my $cache2 = {};
-my $expire;
+my ( $cache1, $cache2, $limit ) = ( {}, {}, 100 );
+my $init;
 
 sub new {
-	my $class = shift;
-	my $s = shift;
+	my ( $class, $s ) = @_;
 	croak 'domain identifier undefined' unless defined $s;
 
 	my $k = join '', $s, $class, $ORIGIN || '';		# cache key
 	my $cache = $$cache1{$k} ||= $$cache2{$k};		# two layer cache
 	return $cache if defined $cache;
 
-	( $cache1, $cache2, $expire ) = ( {}, $cache1, 500 ) unless $expire--;	  # recycle cache
+	( $cache1, $cache2, $limit ) = ( {}, $cache1, 500 ) unless $limit--;	# recycle cache
+
+	init( $init++ ) unless defined $init;			# initialise encoding object
 
 	my $self = bless {}, $class;
 
@@ -137,7 +146,7 @@ numerical escape sequence.
 
 =cut
 
-my $dot = _decode_ascii( pack 'C', 46 );
+my $dot = '.';
 
 sub name {
 	my $self = shift;
@@ -186,7 +195,7 @@ sub xname {
 	return $name unless $name =~ /xn--/;
 
 	my $self = shift;
-	return $self->{xname} ||= UTF8->decode( Net::LibIDN::idn_to_unicode( $name, 'utf-8' ) || $name );
+	return $self->{xname} ||= $utf8->decode( Net::LibIDN::idn_to_unicode( $name, 'utf-8' ) || $name );
 }
 
 
@@ -256,20 +265,22 @@ sub origin {
 
 use vars qw($AUTOLOAD);
 
-sub AUTOLOAD {			## Default method
+sub AUTOLOAD {				## Default method
 	no strict;
 	@_ = ("method $AUTOLOAD undefined");
 	goto &{'Carp::confess'};
 }
 
 
-sub DESTROY { }			## Avoid tickling AUTOLOAD (in cleanup)
+sub DESTROY { }				## Avoid tickling AUTOLOAD (in cleanup)
 
 
-sub _decode_ascii {
+sub _decode_ascii {			## translate ASCII to perl string
 	my $s = shift;
 
-	return pack 'a0 a*', $s, ASCII->decode($s) if ASCII;	# preserve taint
+	my $t = substr $s, 0, 0;				# pre-5.18 taint workaround
+	my $z = length $t;
+	return pack "a* x$z", $ascii->decode($s) if ASCII;
 
 	# partial transliteration for non-ASCII character encodings
 	$s =~ tr
@@ -280,14 +291,15 @@ sub _decode_ascii {
 }
 
 
-sub _encode_ascii {
+sub _encode_ascii {			## translate perl string to ASCII
 	my $s = shift;
-	my $z = substr $s, 0, 0;
 
-	return pack 'a0 a*', $z, Net::LibIDN::idn_to_ascii( $s, 'utf-8' ) || croak 'invalid name'
+	my $t = substr $s, 0, 0;				# pre-5.18 taint workaround
+	my $z = length $t;
+	return pack "a* x$z", Net::LibIDN::idn_to_ascii( $s, 'utf-8' ) || croak 'invalid name'
 			if LIBIDN && $s =~ /[^\000-\177]/;
 
-	return pack 'a0 a*', $z, ASCII->encode($s) if ASCII;	# preserve taint
+	return pack "a* x$z", $ascii->encode($s) if ASCII;
 
 	# partial transliteration for non-ASCII character encodings
 	$s =~ tr
@@ -310,15 +322,20 @@ my %esc = eval {			## precalculated ASCII escape table
 		$table{pack( 'C', $_ )} = pack 'C*', 92, $_;
 	}
 
-	foreach ( 0 .. 32, 127 .. 255 ) {			# \ddd
-		$table{pack( 'C', $_ )} = sprintf '\\%03u', $_;
+	foreach my $n ( 0 .. 32, 127 .. 255 ) {			# \ddd
+		my $codepoint = sprintf( '%03u', $n );
+
+		# partial transliteration for non-ASCII character encodings
+		$codepoint =~ tr [0-9] [\060-\071];
+
+		$table{pack( 'C', $n )} = pack 'C a3', 92, $codepoint;
 	}
 
 	return %table;
 };
 
 
-sub _escape {			## Insert escape sequences in string
+sub _escape {				## Insert escape sequences in string
 	my $s = shift;
 	$s =~ s/([^\055\101-\132\141-\172\060-\071])/$esc{$1}/eg;
 	return $s;
@@ -328,17 +345,21 @@ sub _escape {			## Insert escape sequences in string
 my %unesc = eval {			## precalculated numeric escape table
 	my %table;
 
-	foreach ( 0 .. 255 ) {
-		$table{_encode_ascii sprintf( '%03u', $_ )} = pack 'C', $_;
-	}
+	foreach my $n ( 0 .. 255 ) {
+		my $key = sprintf( '%03u', $n );
 
-	$table{_encode_ascii('092')} = pack 'C*', 92, 92;	# escaped escape
+		# partial transliteration for non-ASCII character encodings
+		$key =~ tr [0-9] [\060-\071];
+
+		$table{$key} = pack 'C', $n;
+		$table{$key} = pack 'C2', 92, $n if $n == 92;	# escaped escape
+	}
 
 	return %table;
 };
 
 
-sub _unescape {			## Remove escape sequences in string
+sub _unescape {				## Remove escape sequences in string
 	my $s = shift;
 	$s =~ s/\134([\060-\071]{3})/$unesc{$1}/eg;		# numeric escape
 	$s =~ s/\134(.)/$1/g;					# character escape
