@@ -1,12 +1,14 @@
 package Net::DNS::RR::OPT;
 
 #
-# $Id: OPT.pm 1101 2013-08-14 14:40:00Z willem $
+# $Id: OPT.pm 1188 2014-04-03 18:54:34Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1101 $)[1]; # Unchanged since 1037
+$VERSION = (qw$LastChangedRevision: 1188 $)[1];
 
-use base Net::DNS::RR;
+
+use strict;
+use base qw(Net::DNS::RR);
 
 =head1 NAME
 
@@ -89,6 +91,7 @@ sub encode {				## overide RR method
 	pack 'C n n C2n n a*', 0, OPT, $size, @xttl, length($data), $data;
 }
 
+
 sub string {				## overide RR method
 	my $self = shift;
 
@@ -113,6 +116,7 @@ sub string {				## overide RR method
 QQ
 }
 
+
 my $warned;
 
 sub class {				## overide RR method
@@ -130,6 +134,7 @@ sub ttl {				## overide RR method
 	return pack 'C2 n', @{$self}{qw(rcode version flags)} if defined wantarray;
 }
 
+
 sub version {
 	my $self = shift;
 
@@ -148,6 +153,7 @@ sub size {
 	}
 }
 
+
 sub rcode {
 	my $self = shift;
 	return $self->{rcode} || 0 unless scalar @_;
@@ -158,6 +164,7 @@ sub rcode {
 	return $val;
 }
 
+
 sub flags {
 	my $self = shift;
 	return $self->{flags} || 0 unless scalar @_;
@@ -166,6 +173,7 @@ sub flags {
 	return $val;
 }
 
+
 sub options {
 	my $self = shift;
 	my $options = $self->{option} || {};
@@ -187,6 +195,7 @@ sub option {
 	}
 }
 
+
 sub defined {
 	my $self = shift;
 
@@ -196,22 +205,23 @@ sub defined {
 	return 0;
 }
 
+
 sub do {				## historical
 	my $self = shift;
 	carp qq[Deprecated method: please use packet->header->do()] unless $warned++;
-	$self->{flags} & 0x8000;
+	$self->{'flags'} & 0x8000;
 }
 
 sub clear_do {				## historical
 	my $self = shift;
 	carp qq[Deprecated method: please use packet->header->do(0)] unless $warned++;
-	$self->{flags} = ( ~0x8000 & $self->{flags} );
+	$self->{'flags'} = ( ~0x8000 & $self->{'flags'} );
 }
 
 sub set_do {				## historical
 	my $self = shift;
 	carp qq[Deprecated method: please use packet->header->do(1)] unless $warned++;
-	$self->{flags} = ( 0x8000 | $self->{flags} );
+	$self->{'flags'} = ( 0x8000 | $self->{'flags'} );
 }
 
 sub ednsversion	  {&version}
@@ -256,6 +266,7 @@ other unpredictable behaviour.
 =head2 version
 
     $version = $rr->version;
+    $rr->version( $version );
 
 The version of EDNS used by this OPT record.
 
@@ -269,7 +280,7 @@ reassembled in the network stack of the originating host.
 
 =head2 rcode
 
-	$extended_rcode   = $packet->header->rcode;
+	$extended_rcode	  = $packet->header->rcode;
 	$incomplete_rcode = $packet->edns->rcode;
 
 The 12 bit extended RCODE. The most significant 8 bits reside in
@@ -310,10 +321,6 @@ changes is cumulative. An option is deleted if the value is undefined.
 
 Copyright (c)2001,2002	RIPE NCC.  Author Olaf M. Kolkman.
 
-Portions Copyright (c)2012 Dick Franks.
-
-Package template (c)2009,2012 O.M.Kolkman and R.W.Franks.
-
 All Rights Reserved
 
 Permission to use, copy, modify, and distribute this software and its
@@ -332,6 +339,10 @@ PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
 THIS SOFTWARE.
 
+Portions Copyright (c)2012 Dick Franks.
+
+Package template (c)2009,2012 O.M.Kolkman and R.W.Franks.
+
 
 =head1 SEE ALSO
 
